/*
     File: TargetGallery.m
 */
 
#import "TargetGallery.h"
#import "TargetView.h"
#import "TargetObject.h"

static const NSTimeInterval kAnimationDelay = 0.03333333;

@interface TargetGallery ()
- (void)moveTargets:(id)ignored;
@end


@implementation TargetGallery

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        _targets = [[NSMutableArray alloc] initWithCapacity:1];
    }
    return self;
}

- (void)dealloc {
    [self stopAnimating];
    [_targets release];
    [super dealloc];
}


#pragma mark NSResponder

- (void)mouseDown:(NSEvent *) event {
    // In edit mode, let super route the event as normal 
    if (self.isEditing) return [super mouseDown:event];

    // Otherwise, a click hitting this view means the user missed. Beep to let them know.
    NSBeep();
}


#pragma mark NSView

- (NSView *)hitTest:(NSPoint)aPoint {
    if (self.isEditing) {
        return self;
    }
    
    // Not in editing mode, let super (NSView) do its normal hit testing.
    return [super hitTest:aPoint];
}


#pragma mark API

@synthesize isEditing = _isEditing;
@synthesize targets = _targets;

- (void)addTarget:(TargetObject *)target {
    [_targets addObject:target];
    [self addSubview:target.view];
}

- (TargetObject *)targetForView:(NSView *)view {
    for (TargetObject *target in _targets) {
        if (target.view == view) return target;
    }
    
    return nil;
}

- (TargetObject *)targetAtPoint:(NSPoint)point {
    return [self targetForView:[super hitTest:point]];
}

- (void)startAnimating {
    // cancel any outstanding animation request before requesting a new one.
    [self stopAnimating];
    [self performSelector:@selector(moveTargets:) withObject:nil afterDelay:kAnimationDelay];
}

- (void)stopAnimating {
    [[self class] cancelPreviousPerformRequestsWithTarget:self];
}

- (void)moveTargets:(id)ignored {
    // Adjust each target by its velocity
    for (TargetObject *target in _targets) {
        if (!NSEqualPoints(NSZeroPoint, target.velocity)) {
            NSRect frame = [target.view frame];
            frame.origin.x += target.velocity.x * kAnimationDelay;
            frame.origin.y += target.velocity.y * kAnimationDelay;
            
            if (!NSIntersectsRect(frame, self.bounds)) {
                // Take horizontal wrap around into consideration.
                if (target.velocity.x < 0) {
                    if (NSMaxX(frame) < NSMinX(self.bounds)) {
                        frame.origin.x = NSMaxX(self.bounds) - 1;
                    }
                } else if (target.velocity.x > 0) {
                    if (NSMinX(frame) > NSMaxX(self.bounds)) {
                        frame.origin.x = NSMinX(self.bounds) - NSWidth(frame) + 1;
                    }
                }
                
                // Take vertical wrap around into consideration.
                if (target.velocity.y < 0) {
                    if (NSMaxY(frame) < NSMinY(self.bounds)) {
                        frame.origin.y = NSMaxY(self.bounds) - 1;
                    }
                } else if (target.velocity.y > 0) {
                    if (NSMinY(frame) > NSMaxY(self.bounds)) {
                        frame.origin.y = NSMinY(self.bounds) - NSHeight(frame) + 1;
                    }
                }
            }
            
            
            [target.view setFrameOrigin:frame.origin];
        }
    }
    
    // Set up calling next frame of the animation.
    [self performSelector:@selector(moveTargets:) withObject:nil afterDelay:kAnimationDelay];
}

@end
